from wkhtmltopdf.views import PDFTemplateResponse, PDFTemplateView


class PDFTemplateViewMixin(PDFTemplateView):
    template_name = "print.html"
    # Send file as attachement. If True render content in the browser.
    show_content_in_browser = True

    def get_cover_template(self):
        return self.cover_template

    def get_header_template(self):
        return self.header_template

    def get_footer_template(self):
        return self.footer_template

    def get_template_names(self):
        return self.template_name

    def render_to_response(self, context, **response_kwargs):
        """
        Returns a PDF response with a template rendered with the given context.
        """
        filename = response_kwargs.pop("filename", None)
        cmd_options = response_kwargs.pop("cmd_options", None)
        cover_template = response_kwargs.pop("cover_template", None)
        header_template = response_kwargs.pop("header_template", None)
        footer_template = response_kwargs.pop("footer_template", None)
        template_name = response_kwargs.pop("template_name", None)

        if issubclass(self.response_class, PDFTemplateResponse):

            if filename is None:
                filename = self.get_filename()

            if cmd_options is None:
                cmd_options = self.get_cmd_options()

            if cover_template is None:
                cover_template = self.get_cover_template()

            if header_template is None:
                header_template = self.get_header_template()

            if footer_template is None:
                footer_template = self.get_footer_template()

            if template_name is None:
                template_name = self.get_template_names()

            return super(PDFTemplateView, self).render_to_response(
                context=context,
                filename=filename,
                show_content_in_browser=self.show_content_in_browser,
                header_template=header_template,
                footer_template=footer_template,
                cmd_options=cmd_options,
                cover_template=cover_template,
                **response_kwargs
            )
        else:
            return super(PDFTemplateView, self).render_to_response(context=context, **response_kwargs)


class PDFPrintView(PDFTemplateViewMixin):
    pass



from functools import update_wrapper
from inspect import isclass

from django.apps import apps
from django.contrib import messages
from django.contrib.admin import AdminSite as BaseAdminSite
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.urls import NoReverseMatch, reverse
from django.utils.text import capfirst
from django.utils.translation import gettext as _
from django.views import View

# from .atomics import AdminMenu
from .settings import bootmin_settings


class AdminSite(BaseAdminSite):

    enable_nav_sidebar = True
    enable_app_index = True

    # Text to put at the end of each page's <title>.
    site_title = bootmin_settings.SITE_TITLE
    # Text to put in each page's <h1>.
    site_header = bootmin_settings.SITE_HEADER
    # Text to put at the top of the admin index page.
    index_title = bootmin_settings.INDEX_TITLE

    # Exclude app from app_list and prevent index
    app_excludes = bootmin_settings.APP_INDEX_EXCLUDES
    app_default_icons = bootmin_settings.DEFAULT_APP_ICONS

    app_index_template = bootmin_settings.APP_INDEX_TEMPLATE
    logout_template = bootmin_settings.LOGOUT_TEMPLATE

    def _build_app_dict(self, request, label=None):
        """
        Build the app dictionary. The optional `label` parameter filters models
        of a specific app.
        """
        app_dict = {}

        if label:
            models = {m: m_a for m, m_a in self._registry.items() if m._meta.app_label == label}
        else:
            models = self._registry

        for model, model_admin in models.items():
            app_label = model._meta.app_label
            app_config = apps.get_app_config(app_label)

            if app_label in self.app_excludes:
                continue

            # Check whether user has any perm for this module.
            # If so, add the module to the model_list.
            has_module_perms = model_admin.has_module_permission(request)
            if not has_module_perms:
                continue

            perms = model_admin.get_model_perms(request)
            if True not in perms.values():
                continue

            info = (app_label, model._meta.model_name)
            model_icon = getattr(model, "icon", None)
            if model_icon is None:
                if model._meta.model_name == "user":
                    model_icon = "account-circle-outline"
                else:
                    model_icon = "text-box-outline"

            model_dict = {
                "name": capfirst(model._meta.verbose_name_plural),
                "icon": model_icon,
                "object_name": model._meta.object_name,
                "perms": perms,
                "admin_url": None,
                "add_url": None,
            }
            if perms.get("change") or perms.get("view"):
                model_dict["view_only"] = not perms.get("change")
                try:
                    model_dict["admin_url"] = reverse("admin:%s_%s_changelist" % info, current_app=self.name)
                except NoReverseMatch:
                    pass
            if perms.get("add"):
                try:
                    model_dict["add_url"] = reverse("admin:%s_%s_add" % info, current_app=self.name)
                except NoReverseMatch:
                    pass

            if app_label in app_dict:
                app_dict[app_label]["models"].append(model_dict)
            else:
                icon = getattr(app_config, "icon", None)
                if icon is None:
                    icon = self.app_default_icons.get(app_label, "apps")
                app_dict[app_label] = {
                    "name": app_config.verbose_name,
                    "icon": icon,
                    "app_label": app_label,
                    "app_url": reverse(
                        "admin:app_list",
                        kwargs={"app_label": app_label},
                        current_app=self.name,
                    ),
                    "has_module_perms": has_module_perms,
                    "models": [model_dict],
                }

        if label:
            return app_dict.get(label)
        return app_dict

    def get_app_list(self, request):
        """
        Return a sorted list of all the installed apps that have been
        registered in this site.
        """
        app_dict = self._build_app_dict(request)

        # Sort the apps alphabetically.
        app_list = sorted(app_dict.values(), key=lambda x: x["name"].lower())

        # Show app index base on app_config
        # Sort the models alphabetically within each app.
        valid_app_list = []
        for app in app_list:
            app_config = apps.get_app_config(app["app_label"])
            show_app_index = getattr(app_config, "show_app_index", True)
            if show_app_index:
                app["models"].sort(key=lambda x: x["name"])
                valid_app_list.append(app)
        return valid_app_list

    def each_context(self, request):
        context = super().each_context(request)
        context["is_app_index_enabled"] = self.enable_app_index
        # admin_menu = AdminMenu(attrs={"id": "adminMenu"})
        # context["admin_menu"] = admin_menu.render(context={"request": request})
        return context

    def app_index(self, request, app_label, extra_context=None):
        if not self.enable_app_index:
            messages.error(request, _("The app index page is disabled."))
            return redirect("admin:index")

        app_dict = self._build_app_dict(request, app_label)
        if not app_dict:
            messages.error(request, _("The requested admin page does not exist."))
            return redirect("admin:index")

        # Sort the models alphabetically within each app.
        app_dict["models"].sort(key=lambda x: x["name"])
        context = {
            **self.each_context(request),
            "title": _("%(app)s Administration") % {"app": app_dict["name"]},
            "subtitle": None,
            "app_list": [app_dict],
            "app_label": app_label,
            **(extra_context or {}),
        }
        template_names = [
            "admin/%s/app_index.html" % app_label,
            "admin/%s.html" % app_label,
            "admin/app_index.html",
        ]
        request.current_app = self.name
        return TemplateResponse(
            request,
            template_names,
            context,
        )

    def get_urls(self):
        # Since this module gets imported in the application's root package,
        # it cannot import models from other applications at the module level,
        # and django.contrib.contenttypes.views imports ContentType.
        from django.contrib.contenttypes import views as contenttype_views
        from django.urls import include, path, re_path

        from django_hookup import core as hookup

        def wrap(view, cacheable=False):
            def wrapper(*args, **kwargs):
                return self.admin_view(view, cacheable)(*args, **kwargs)

            wrapper.admin_site = self
            return update_wrapper(wrapper, view)

        # Get registered custom admin view
        funcs = hookup.get_hooks("REGISTER_ADMIN_VIEW")
        urlpatterns = []
        for func in funcs:
            url_path, view, name = func()
            if isclass(view):
                if not issubclass(view, View):
                    raise ImproperlyConfigured("%s must be subclass of View" % view)
                route = path(url_path, wrap(view.as_view()), name=name)
            else:
                route = path(url_path, wrap(view), name=name)
            urlpatterns.append(route)

        # Admin-site-wide views.
        urlpatterns += [
            path("", wrap(self.index), name="index"),
            path("login/", self.login, name="login"),
            path("logout/", wrap(self.logout), name="logout"),
            path("password_change/", wrap(self.password_change, cacheable=True), name="password_change"),
            path(
                "password_change/done/",
                wrap(self.password_change_done, cacheable=True),
                name="password_change_done",
            ),
            path("autocomplete/", wrap(self.autocomplete_view), name="autocomplete"),
            path("jsi18n/", wrap(self.i18n_javascript, cacheable=True), name="jsi18n"),
            path(
                "r/<int:content_type_id>/<path:object_id>/",
                wrap(contenttype_views.shortcut),
                name="view_on_site",
            ),
        ]

        # Add in each model's views, and create a list of valid URLS for the
        # app_index
        valid_app_labels = []
        for model, model_admin in self._registry.items():
            path_name = (model._meta.app_label, model._meta.model_name)

            # Attach model_admin to selected app_label
            # attach_to_app = getattr(model_admin, "attach_to_app", None)
            # if attach_to_app is not None:
            #     path_name = (attach_to_app, model._meta.model_name)

            urlpatterns += [
                path("%s/%s/" % path_name, include(model_admin.urls)),
            ]
            if model._meta.app_label not in valid_app_labels:
                valid_app_labels.append(model._meta.app_label)

        # If there were ModelAdmins registered, we should have a list of app
        # labels for which we need to allow access to the app_index view,

        if valid_app_labels:
            regex = r"^(?P<app_label>" + "|".join(valid_app_labels) + ")/$"
            urlpatterns += [
                re_path(regex, wrap(self.app_index), name="app_list"),
            ]

        if self.final_catch_all_view:
            urlpatterns.append(re_path(r"(?P<url>.*)$", wrap(self.catch_all_view)))

        return urlpatterns



from django.contrib import admin, messages
from django.contrib.admin.templatetags.admin_urls import admin_urlname
from django.contrib.auth import get_permission_codename
from django.shortcuts import get_object_or_404, redirect
from django.template.loader import render_to_string
from django.template.response import TemplateResponse
from django.urls import path, reverse
from django.utils.translation import ugettext_lazy as _
from django.views.generic import FormView, TemplateView

from .settings import bootmin_settings


class AdminView(TemplateView):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def has_permission(self, request=None, obj=None):
        return True

    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(admin.site.each_context(self.request))
        return context

    def has_action_permission(self, request, action):
        opts = self.opts
        codename = get_permission_codename(action, opts)
        return request.user.has_perm("%s.%s" % (opts.app_label, codename))


class AdminFormView(FormView):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def has_action_permission(self, request, action):
        opts = self.opts
        codename = get_permission_codename(action, opts)
        return request.user.has_perm("%s.%s" % (opts.app_label, codename))

    def has_permission(self, request=None, obj=None):
        return True

    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(admin.site.each_context(self.request))
        return context


class BaseModelAdmin(admin.ModelAdmin):

    confirmation_template = "admin/confirmation.html"

    def has_action_permission(self, request, action):
        opts = self.opts
        codename = get_permission_codename(action, opts)
        return request.user.has_perm("%s.%s" % (opts.app_label, codename))

    def get_changelist_url(self):
        return reverse(admin_urlname(self.opts, "changelist"))

    def get_add_url(self):
        return reverse(admin_urlname(self.opts, "add"))

    def get_change_url(self, object_id):
        return reverse(admin_urlname(self.opts, "change"), args=(object_id,))

    def get_delete_url(self, object_id):
        return reverse(admin_urlname(self.opts, "delete"), args=(object_id,))

    def get_history_url(self, object_id):
        return reverse(admin_urlname(self.opts, "delete"), args=(object_id,))

    def get_object_buttons_childs(self, request, obj):
        childs = []
        return childs

    def get_object_buttons_template(self, request, obj):
        return "admin/object_buttons.html"

    @admin.display(description="")
    def object_buttons(self, obj):
        childs = self.get_object_buttons_childs(self.request, obj)
        template = self.get_object_buttons_template(self.request, obj)
        context = {"request": self.request, "childs": childs, "object": obj}
        return render_to_string(template, context)

    def changelist_view(self, request, extra_context=None):
        self.request = request
        return super().changelist_view(request, extra_context)

    def confirmation_view(self, request, extra_context={}):
        context = {
            "request": request,
            "opts": self.opts,
            **self.admin_site.each_context(request),
            **extra_context,
        }
        return TemplateResponse(
            request,
            self.confirmation_template,
            context=context,
        )


class AdminPrintViewMixin(BaseModelAdmin):
    print_options = bootmin_settings.PRINT_OPTIONS
    print_filename = None
    print_template = None
    header_template = None
    footer_template = None
    cover_template = None
    print_view_class = bootmin_settings.PRINT_VIEW_CLASS

    def get_print_template(self):
        return self.print_template or [
            "admin/%s/%s/print.html" % (self.opts.app_label, self.opts.model_name),
            "admin/%s/%s_print.html" % (self.opts.app_label, self.opts.model_name),
            "admin/%s_%s_print.html" % (self.opts.app_label, self.opts.model_name),
            "admin/%s_print.html" % (self.opts.model_name),
            "admin/print.html",
        ]

    def get_print_filename(self):
        return self.print_filename or "%s_print" % self.opts.model_name

    def get_print_options(self):
        return self.print_options

    def get_cover_template(self):
        return self.cover_template

    def get_header_template(self):
        return self.header_template

    def get_footer_template(self):
        return self.footer_template

    def get_print_context(self, **kwargs):
        kwargs.update({"opts": self.opts})
        return kwargs

    def get_print_view_class(self):
        return self.print_view_class

    def get_print_url(self, object_id):
        return reverse(admin_urlname(self.opts, "print"), args=(object_id,))

    def has_print_permission(self, request, obj=None):
        return self.has_view_permission(request)

    def print_view(self, request, pk, *args, **kwargs):
        self.request = request
        obj = self.get_object(request, object_id=pk)

        if not self.has_print_permission(request, obj):
            messages.error(request, _("You don't have permission to print this object"))
            try:
                return redirect(self.get_inspect_url(object_id=obj.id))
            except Exception:
                return redirect(self.get_change_url(object_id=obj.id))

        init_kwargs = dict(
            cmd_options=self.get_print_options(),
            filename=self.get_print_filename(),
            template_name=self.get_print_template(),
            cover_template=self.get_cover_template(),
            header_template=self.get_header_template(),
            footer_template=self.get_footer_template(),
            extra_context=self.get_print_context(object=obj, request=request),
        )
        view_class = self.get_print_view_class()
        return view_class.as_view(**init_kwargs)(request, pk, *args, **kwargs)

    def get_urls(self):
        super_urls = super().get_urls()
        urls = [
            path(
                "<int:pk>/print/",
                self.print_view,
                name="%s_%s_print" % (self.opts.app_label, self.opts.model_name),
            ),
        ]
        urls += super_urls
        return urls


class AdminInspectionMixin(BaseModelAdmin):
    inspect_enabled = True
    inspect_template = None
    inspect_title = None

    def get_inspect_url(self, object_id):
        return reverse(admin_urlname(self.opts, "inspect"), args=(object_id,))

    def get_object_buttons_childs(self, request, obj):
        childs = super().get_object_buttons_childs(request, obj)
        if self.inspect_enabled:
            childs += [
                {
                    "url": reverse(admin_urlname(self.opts, "inspect"), args=(obj.id,)),
                    "classes": "btn btn-sm btn-primary",
                    "label": _("Inspect"),
                }
            ]
        return childs

    def get_inspect_title(self, obj):
        return self.inspect_title or _("Inspecting %s") % obj

    def get_inspect_context(self, **kwargs):
        kwargs.update(
            {
                "opts": self.opts,
                "object": self.object,
                "title": self.get_inspect_title(self.object),
                "available_apps": admin.site.get_app_list(self.request),
                "has_change_permission": self.has_change_permission(self.request, self.object),
                "has_add_permission": self.has_add_permission(self.request),
                "has_delete_permission": self.has_delete_permission(self.request, self.object),
                "has_view_permission": self.has_view_permission(self.request, self.object),
                **self.admin_site.each_context(self.request),
            }
        )
        has_print_permission = getattr(self, "has_print_permission", None)
        if has_print_permission:
            kwargs["has_print_permission"] = has_print_permission(self.request, self.object)
        return kwargs

    def get_inspect_template(self):
        return self.inspect_template or [
            "admin/%s/%s/inspect.html" % (self.opts.app_label, self.opts.model_name),
            "admin/%s/%s_inspect.html" % (self.opts.app_label, self.opts.model_name),
            "admin/%s_%s_inspect.html" % (self.opts.app_label, self.opts.model_name),
            "admin/%s_inspect.html" % (self.opts.model_name),
            "admin/inspect.html",
        ]

    def inspect_view(self, request, pk, **kwargs):
        self.request = request
        self.object_id = pk
        self.object = self.get_object(request, object_id=pk)
        return TemplateResponse(
            request=request,
            template=self.get_inspect_template(),
            context=self.get_inspect_context(**kwargs),
        )

    def change_view(self, request, object_id, form_url="", extra_context=None):
        if object_id:
            self.object = get_object_or_404(self.model, pk=object_id)
            if self.inspect_enabled and not self.has_change_permission(request, self.object):
                return redirect(self.get_inspect_url(object_id))
        return super().change_view(request, object_id, form_url=form_url, extra_context=extra_context)

    def get_urls(self):
        super_urls = super().get_urls()
        urls = []
        if self.inspect_enabled:
            urls.append(
                path(
                    "<int:pk>/inspect/",
                    self.admin_site.admin_view(self.inspect_view),
                    name="%s_%s_inspect" % (self.opts.app_label, self.opts.model_name),
                ),
            )
        urls += super_urls
        return urls


class AdminReadOnlyMixin(BaseModelAdmin):
    def __init__(self, model, admin_site):
        super().__init__(model, admin_site)

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        return super().get_urls()


class ModelAdminMixin(AdminInspectionMixin):
    pass



from django.contrib.admin.filters import SimpleListFilter
from django.utils.translation import gettext_lazy as _


class PreFilteredListFilter(SimpleListFilter):

    # Either set this or override .get_default_value()
    default_value = None

    no_filter_value = "all"
    no_filter_name = _("All")

    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = None

    # Parameter for the filter that will be used in the URL query.
    parameter_name = None

    def get_default_value(self):
        if self.default_value is not None:
            return self.default_value
        raise NotImplementedError(
            "Either the .default_value attribute needs to be set or "
            "the .get_default_value() method must be overridden to "
            "return a URL query argument for parameter_name."
        )
